@@ -33,6 +33,8 @@
 #include "goafacebookprovider.h"
 #include "goaimapsmtpprovider.h"
 #include "goaowncloudprovider.h"
+#include "goayahooprovider.h"
+#include "goatwitterprovider.h"
 #include "goaflickrprovider.h"
 #include "goawindowsliveprovider.h"
 
@@ -51,6 +53,19 @@
  * #GoaProvider is the base type for all providers.
  */
 
+struct _GoaProviderPrivate
+{
+  GVariant *preseed_data;
+};
+
+enum {
+  PROP_0,
+  PROP_PRESEED_DATA,
+  NUM_PROPERTIES
+};
+
+static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
+
 static gboolean goa_provider_ensure_credentials_sync_real (GoaProvider   *provider,
                                                            GoaObject     *object,
                                                            gint          *out_expires_in,
@@ -77,21 +92,120 @@ static guint goa_provider_get_credentials_generation_real (GoaProvider *provider
 static GIcon *goa_provider_get_provider_icon_default (GoaProvider *provider,
                                                       GoaObject   *object);
 
+#define GOA_PROVIDER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOA_TYPE_PROVIDER, GoaProviderPrivate))
+
 G_DEFINE_ABSTRACT_TYPE (GoaProvider, goa_provider, G_TYPE_OBJECT);
 
 static void
-goa_provider_init (GoaProvider *client)
+goa_provider_get_property (GObject *object,
+                           guint property_id,
+                           GValue *value,
+                           GParamSpec *pspec)
 {
+    GoaProvider *self = GOA_PROVIDER (object);
+
+    switch (property_id) {
+    case PROP_PRESEED_DATA:
+        g_value_set_variant (value, self->priv->preseed_data);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
+static void
+goa_provider_set_property (GObject *object,
+                           guint property_id,
+                           const GValue *value,
+                           GParamSpec *pspec)
+{
+    GoaProvider *self = GOA_PROVIDER (object);
+
+    switch (property_id) {
+    case PROP_PRESEED_DATA:
+        goa_provider_set_preseed_data (self, g_value_get_variant (value));
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
+static void
+goa_provider_dispose (GObject *object)
+{
+  GoaProvider *provider = GOA_PROVIDER (object);
+
+  g_clear_pointer (&provider->priv->preseed_data, g_variant_unref);
+
+  G_OBJECT_CLASS (goa_provider_parent_class)->dispose (object);
+}
+
+static void
+goa_provider_init (GoaProvider *provider)
+{
+  provider->priv = GOA_PROVIDER_GET_PRIVATE (provider);
 }
 
 static void
 goa_provider_class_init (GoaProviderClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  g_type_class_add_private (klass, sizeof (GoaProviderPrivate));
+
+  object_class->set_property = goa_provider_set_property;
+  object_class->get_property = goa_provider_get_property;
+  object_class->dispose = goa_provider_dispose;
+
   klass->build_object = goa_provider_build_object_real;
   klass->ensure_credentials_sync = goa_provider_ensure_credentials_sync_real;
   klass->show_account = goa_provider_show_account_real;
   klass->get_credentials_generation = goa_provider_get_credentials_generation_real;
   klass->get_provider_icon = goa_provider_get_provider_icon_default;
+
+/**
+ * GoaProvider:preseed-data
+ *
+ * An #GVariant of type a{sv} storing any information already collected that
+ * can be useful when creating a new account. For instance, this can be useful
+ * to reuse the HTTP cookies from an existing browser session to skip the
+ * prompt for username and password in the OAuth2-based providers by passing
+ * a #GVariant with the following contents:
+ *
+ * <informalexample>
+ * <programlisting>
+ * {
+ *   "cookies": [
+ *     {
+ *       "domain": "example.com",
+ *       "name": "LSID",
+ *       "value": "asdfasdfasdf"
+ *     },
+ *     {
+ *       "domain": "accounts.example.com",
+ *       "name": "SSID",
+ *       "value": "asdfasdfasdf"
+ *     }
+ *   ]
+ * }
+ * </programlisting>
+ * </informalexample>
+ *
+ * Unknown or unsupported keys will be ignored by providers.
+ */
+  properties[PROP_PRESEED_DATA] =
+    g_param_spec_variant ("preseed-data",
+        "Collected data to pre-seed account creation",
+        "A a{sv} #GVariant containing a provider-type specific set of data that"
+        "can be useful during account creation (eg. http cookies from an existing"
+        "browser session or the entrypoint url for self-hosted services).",
+        G_VARIANT_TYPE_VARDICT,
+        NULL,
+        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, NUM_PROPERTIES, properties);
 }
 
 /**
@@ -185,6 +299,8 @@ goa_provider_get_provider_icon_default (GoaProvider *provider,
  * Returns: A #GoaProviderGroup.
  *
  * Since: 3.8
+ *
+ * Deprecated: 3.10: Use goa_provider_get_provider_features() instead.
  */
 GoaProviderGroup
 goa_provider_get_provider_group (GoaProvider *provider)
@@ -193,6 +309,25 @@ goa_provider_get_provider_group (GoaProvider *provider)
   return GOA_PROVIDER_GET_CLASS (provider)->get_provider_group (provider);
 }
 
+/**
+ * goa_provider_get_provider_features:
+ * @provider: A #GoaProvider.
+ *
+ * Get the features bitmask (eg. %GOA_PROVIDER_FEATURE_CHAT|%GOA_PROVIDER_FEATURE_CONTACTS)
+ * supported by the provider.
+ *
+ * Returns: The #GoaProviderFeatures bitmask with the provided features.
+ *
+ * Since: 3.10
+ */
+GoaProviderFeatures
+goa_provider_get_provider_features (GoaProvider *provider)
+{
+  g_return_val_if_fail (GOA_IS_PROVIDER (provider), GOA_PROVIDER_FEATURE_INVALID);
+  g_return_val_if_fail (GOA_PROVIDER_GET_CLASS (provider)->get_provider_features != NULL, GOA_PROVIDER_FEATURE_INVALID);
+  return GOA_PROVIDER_GET_CLASS (provider)->get_provider_features (provider);
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 /**
@@ -683,6 +818,12 @@ ensure_builtins_loaded (void)
 #ifdef GOA_KERBEROS_ENABLED
       type = GOA_TYPE_KERBEROS_PROVIDER;
 #endif
+#ifdef GOA_YAHOO_ENABLED
+      type = GOA_TYPE_YAHOO_PROVIDER;
+#endif
+#ifdef GOA_TWITTER_ENABLED
+      type = GOA_TYPE_TWITTER_PROVIDER;
+#endif
 
       type = type; /* silence -Wunused-but-set-variable */
 
@@ -758,6 +899,44 @@ goa_provider_get_all (void)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+/**
+ * goa_provider_set_preseed_data:
+ * @provider: The #GoaProvider
+ * @preseed_data: A #GVariant of type a{sv}
+ *
+ * Sets the #GoaProvider:preseed-data property to feed any information already
+ * collected that can be useful when creating a new account.
+ *
+ * If the @preseed_data #GVariant is floating, it is consumed to allow
+ * 'inline' use of the g_variant_new() family of functions.
+ */
+void
+goa_provider_set_preseed_data (GoaProvider *provider,
+                               GVariant    *preseed_data)
+{
+  g_clear_pointer (&provider->priv->preseed_data, g_variant_unref);
+  if (preseed_data != NULL)
+    provider->priv->preseed_data = g_variant_ref_sink (preseed_data);
+  g_object_notify (G_OBJECT (provider), "preseed-data");
+}
+
+/**
+ * goa_provider_get_preseed_data:
+ * @provider: The #GoaProvider
+ *
+ * Gets the #GVariant set through the #GoaProvider:preseed-data property.
+ *
+ * Returns: (transfer none): A #GVariant that is known to be valid until
+ *   the property is overridden or the provider freed.
+ */
+GVariant *
+goa_provider_get_preseed_data (GoaProvider *provider)
+{
+  return provider->priv->preseed_data;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 /**
  * SECTION:goautil
  * @title: Utilities
